package ru.vyatsu.qr_access_api.booking.repository

import org.springframework.dao.EmptyResultDataAccessException
import org.springframework.jdbc.core.JdbcTemplate
import org.springframework.jdbc.support.GeneratedKeyHolder
import org.springframework.stereotype.Repository
import org.springframework.transaction.annotation.Transactional
import ru.vyatsu.apis.NotFoundException
import ru.vyatsu.qr_access_api.booking.repository.entity.RentWithEmail
import java.sql.Date
import java.sql.Statement
import java.sql.Time
import java.sql.Timestamp
import java.time.LocalDate
import java.time.LocalDateTime
import java.time.LocalTime
import java.util.*

private const val ADD_BOOKING_INFO_QUERY =
    "insert into rents(id, start_time, end_time, client_id, date, door_id, qr_code, payed, date_created) values (?, ? ,?, ?, ?, ?, ?, false, CURRENT_TIMESTAMP) RETURNING id"

private const val CREATE_NEW_CLIENT =
    "insert into clients(id, email, email_is_confirmed) values (?, ?, false) RETURNING id"

private const val FIND_CLIENT_BY_EMAIL = "select id from clients where email = ?"

private const val MARK_RENT_AS_PAYED_QUERY = "update rents set payed=true where id = ?"

@Repository
class BookingRepository(val template: JdbcTemplate) {

    @Transactional
    fun book(doorId: String, date: LocalDate, startTime: LocalTime, endTime: LocalTime, clientEmail: String): String {
        val clientIdHolder = GeneratedKeyHolder()
        val rentIdHolder = GeneratedKeyHolder()
        var clientId: String? = null
        try {
            clientId = template.queryForObject(
                FIND_CLIENT_BY_EMAIL,
                { rs, _ -> rs.getString("id") },
                clientEmail
            )
        } catch (_: EmptyResultDataAccessException) {
        }

        if (clientId == null) {
            template.update({
                val stmt = it.prepareStatement(CREATE_NEW_CLIENT, Statement.RETURN_GENERATED_KEYS)
                stmt.setString(1, UUID.randomUUID().toString())
                stmt.setString(2, clientEmail)
                stmt
            }, clientIdHolder)
            clientId = clientIdHolder.getKeyAs(String::class.java)
        }

        if (clientId == null) throw RuntimeException("clientId is null even after insert, booking cannot be continued")

        val insertedRows = template.update({
            val stmt = it.prepareStatement(ADD_BOOKING_INFO_QUERY, Statement.RETURN_GENERATED_KEYS)
            stmt.setString(1, UUID.randomUUID().toString())
            stmt.setTime(2, Time.valueOf(startTime))
            stmt.setTime(3, Time.valueOf(endTime))
            stmt.setString(4, clientId)
            stmt.setDate(5, Date.valueOf(date))
            stmt.setString(6, doorId)
            stmt.setString(7, UUID.randomUUID().toString())
            stmt
        }, rentIdHolder)

        val insertedRentId = rentIdHolder.getKeyAs(String::class.java)
        if (insertedRows <= 0 || insertedRentId == null) throw RuntimeException(
            "Inserted rows number is invalid: %d. Have to be more then 0".format(
                insertedRows
            )
        )

        return insertedRentId
    }

    fun findRentWithClientById(id: String): RentWithEmail {
        return template.queryForObject(
            "select r.qr_code, c.email from rents r join clients c on (c.id = r.client_id) where r.id = ?",
            { rs, _ -> RentWithEmail(rs.getString("qr_code"), rs.getString("email")) },
            id
        ) ?: throw NotFoundException("Cannot find rent with id %s".format(id))
    }

    fun markBookingPayed(rentId: String) {
        val updatedAmount = template.update {
            val stmt = it.prepareStatement(MARK_RENT_AS_PAYED_QUERY)
            stmt.setString(1, rentId)
            stmt
        }

        if (updatedAmount == 0)
            throw NotFoundException("Cannot find rent with id %s".format(rentId))
    }

    fun deleteRentByDateCreatedLessThen(dateTime: LocalDateTime): Int {
        return template.update {
            val stmt = it.prepareStatement("delete from rents where date_created <= ? and payed = false")
            stmt.setTimestamp(1, Timestamp.valueOf(dateTime))
            stmt
        }
    }
}