package ru.vyatsu.qr_access_api.slots.repository

import org.springframework.jdbc.core.JdbcTemplate
import org.springframework.stereotype.Repository
import ru.vyatsu.qr_access_api.slots.repository.entity.*

const val FIND_DOORS_SCHEDULE_BY_PARTNER_ID =
    "select d.id, d.description, d.count, s.date, s.start_time, s.end_time from oauth2_registered_client u join doors d on (d.unit_id = u.client_id) join schedule s on (d.id = s.door_id) where u.partner_id = ?"
const val FIND_RENT_DOORS_BY_PARTNER_ID =
    "select d.id, r.date, r.start_time, r.end_time from oauth2_registered_client u join doors d on (d.unit_id = u.client_id) join rents r on (r.door_id = d.id) where u.partner_id = ? order by r.start_time asc"

@Repository
class SlotRepository(private val jdbc: JdbcTemplate) {
    fun findAllDoorsWithScheduleByPartnerId(partnerId: String): Collection<DoorWithSchedule> {
        val doors: MutableMap<String, DoorWithSchedule> = mutableMapOf()
        jdbc.query({
            val stmt = it.prepareStatement(FIND_DOORS_SCHEDULE_BY_PARTNER_ID)
            stmt.setString(1, partnerId)
            stmt
        }, { rs ->
            val id = rs.getString("id")
            doors[id]?.also {
                it.schedule.add(
                    ScheduleEntry(
                        rs.getTime("start_time").toLocalTime(),
                        rs.getTime("end_time").toLocalTime(),
                        rs.getDate("date").toLocalDate()
                    )
                )
            } ?: run {
                doors[id] = DoorWithSchedule(
                    id, rs.getString("description"), rs.getInt("count"), mutableListOf(
                        ScheduleEntry(
                            rs.getTime("start_time").toLocalTime(),
                            rs.getTime("end_time").toLocalTime(),
                            rs.getDate("date").toLocalDate()
                        )
                    )
                )
            }
        })
        return doors.values
    }

    fun findRentDateTimesByPartnerId(partnerId: String): DoorRent {
        val doors = DoorRent(mutableMapOf())
        jdbc.query({
            val stmt = it.prepareStatement(FIND_RENT_DOORS_BY_PARTNER_ID)
            stmt.setString(1, partnerId)
            stmt
        }, { rs ->
            val id = rs.getString("id")
            val date = rs.getDate("date").toLocalDate()
            doors.dates[id]?.also { d ->
                d.rentDate[date]?.also { times ->
                    times.add(RentTime(rs.getTime("start_time").toLocalTime(), rs.getTime("end_time").toLocalTime()))
                } ?: run {
                    d.rentDate[date] = mutableListOf(
                        RentTime(
                            rs.getTime("start_time").toLocalTime(),
                            rs.getTime("end_time").toLocalTime()
                        )
                    )
                }
            } ?: run {
                doors.dates[id] = RentDate(
                    mutableMapOf(
                        date to mutableListOf(
                            RentTime(
                                rs.getTime("start_time").toLocalTime(),
                                rs.getTime("end_time").toLocalTime()
                            )
                        )
                    )
                )
            }
        })
        return doors
    }
}